'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const MICRO = '\u00b5';

function error(message) {
  return new TypeError(message);
}
function errorInvalidOption(option) {
  return error("Invalid \"" + option + "\" option");
}
function errorOptionRemoved(removed, replacement) {
  return error("Option \"" + removed + "\" has been removed. Use \"" + replacement + "\" option instead.");
}

function isType(type) {
  return value => {
    return typeof value === type;
  };
}
const isFunction = isType('function');
const isObject = isType('object');
const {
  isArray
} = Array;
const isNumber = isType('number');
const {
  isFinite: isFiniteNumber
} = Number;

const exa = {
  pre: 'E',
  exp: 6
};
const peta = {
  pre: 'P',
  exp: 5
};
const tera = {
  pre: 'T',
  exp: 4
};
const giga = {
  pre: 'G',
  exp: 3
};
const mega = {
  pre: 'M',
  exp: 2
};
const kilo = {
  pre: 'k',
  exp: 1
};
const milli = {
  pre: 'm',
  exp: -1
};
const micro = {
  pre: MICRO,
  exp: -2
};
const nano = {
  pre: 'n',
  exp: -3
};
const pico = {
  pre: 'p',
  exp: -4
};
const femto = {
  pre: 'f',
  exp: -5
};
const atto = {
  pre: 'a',
  exp: -6
};

const {
  pow
} = Math;

function transformFindUnitArray(units, base) {
  return units.map(_ref => {
    let {
      pre,
      exp
    } = _ref;
    return {
      pre,
      div: pow(base, exp)
    };
  });
}
function sortFindUnitArray(units, base) {
  return transformFindUnitArray(units, base).sort((a, b) => b.div - a.div);
}
const unity = {
  pre: '',
  div: 1
};
// IMPORTANT TO DEVELOPERS
// this array has to be sorted from bigger to smaller unit!
// while using this array internally we skip sorting it as it is expected to be already sorted
// unlike arrays provided by the user which will be sorted
const defaultBase1000FormatFindItems = [exa, peta, tera, giga, mega, kilo, {
  exp: 0,
  pre: ''
}, milli, micro, nano, pico, femto, atto];
function createFindItems(find) {
  if (!find) {
    return transformFindUnitArray(defaultBase1000FormatFindItems, 1000);
  }
  if (isNumber(find)) {
    return transformFindUnitArray(defaultBase1000FormatFindItems, find);
  }
  if (isArray(find)) {
    return sortFindUnitArray(find, 1000);
  }
  const {
    find: items,
    base
  } = find;
  const baseAsNumber = base !== null && base !== void 0 ? base : 1000;
  if (items) {
    return sortFindUnitArray(items, baseAsNumber);
  }
  return transformFindUnitArray(defaultBase1000FormatFindItems, baseAsNumber);
}

function createUnitFinder(find) {
  // return wrapped function if it's a function
  if (isFunction(find)) {
    return value => {
      const result = find(value);
      if (!isObject(result)) {
        throw error(result + " is not a valid return value for \"find\" option");
      }
      const {
        div
      } = result;
      if (!isNumber(div) || !isFiniteNumber(div) || !div) {
        throw error(div + " is not a valid divider");
      }
      return result;
    };
  }
  const findItems = createFindItems(find);
  return value => {
    if (!value) {
      return unity;
    }
    const absolute = Math.abs(value);
    const lastIndex = findItems.length - 1;
    for (let i = 0; i < lastIndex; i++) {
      const item = findItems[i];
      if (absolute >= item.div) {
        return item;
      }
    }
    return lastIndex >= 0 ? findItems[lastIndex] : unity;
  };
}

function createFormatOutput(output) {
  if (output == null) {
    return (value, pre, unit) => {
      const wholeUnit = "" + pre + unit;
      const spacedUnit = wholeUnit ? " " + wholeUnit : '';
      return "" + value + spacedUnit;
    };
  }
  if (!isFunction(output)) {
    throw errorInvalidOption('output');
  }
  return output;
}

function createRounderWith(dec, fixed) {
  if (!isFiniteNumber(dec) || dec < 0) {
    throw error("Can't create round function with " + dec + " decimal.");
  }
  if (fixed) {
    return num => num.toFixed(dec);
  }
  const roundMultiplier = pow(10, dec);
  return num => Math.round(num * roundMultiplier) / roundMultiplier;
}
function createRounder(round) {
  // return default rounder if no "round" option
  if (round == null) {
    return createRounderWith(2);
  }
  // return user option if it's a function
  if (isFunction(round)) {
    return round;
  }
  // create rounder with number of decimals provided if it's a number
  if (isNumber(round)) {
    return createRounderWith(round);
  }
  if (!isObject(round)) {
    throw errorInvalidOption('round');
  }
  // get data from advanced round object
  const {
    dec,
    fixed
  } = round;
  // return rounder with provided options
  return createRounderWith(dec ? +dec : 2, fixed);
}

/**
 * Create a new formatter
 *
 * @param options create formatter options
 * @returns formatter
 */
function createFormatter(options) {
  if (options === void 0) {
    options = {};
  }
  if ('table' in options) {
    throw errorOptionRemoved('table', 'find');
  }
  if ('dec' in options) {
    throw errorOptionRemoved('dec', 'round');
  }
  if ('fixed' in options) {
    throw errorOptionRemoved('fixed', 'round');
  }
  const {
    unit,
    find,
    round,
    output
  } = options;
  const getUnit = isFunction(unit) ? unit : () => unit !== null && unit !== void 0 ? unit : '';
  const findUnit = createUnitFinder(find);
  const roundNum = createRounder(round);
  const formatOutput = createFormatOutput(output);
  return value => {
    const item = findUnit(value);
    const {
      pre,
      div: divisor
    } = item;
    const divided = value / divisor;
    const rounded = roundNum(divided);
    const result = formatOutput(rounded, pre, getUnit(divided, rounded, pre));
    return "" + result;
  };
}

function format(value, options) {
  return createFormatter(options)(value);
}

/**
 * Create a extractPre function
 *
 * @param unit parser "unit" options
 * @returns extractPre function
 */
function createExtractPre(unit) {
  if (unit) {
    return wholeUnit => {
      if (wholeUnit.endsWith(unit)) {
        const unitLength = unit.length;
        return wholeUnit.slice(0, -unitLength);
      }
      return wholeUnit;
    };
  }
  return wholeUnit => wholeUnit;
}

const defaultBase1000ParseFindItems = [exa, peta, tera, giga, {
  pre: 'meg',
  exp: 2
}, mega, {
  pre: 'K',
  exp: 1
}, kilo, milli, {
  pre: 'u',
  exp: -2
}, micro, nano, pico, femto, atto];
/**
 * transforms find items into a multiplier find table
 *
 * @param items find items
 * @param base base
 * @returns find multiplier table
 */
function transformItems(items, base) {
  return items.map(_ref => {
    let {
      pre,
      exp
    } = _ref;
    const multiplier = pow(base, exp);
    if (!isFiniteNumber(multiplier) || multiplier <= 0) {
      throw error(base + " to the power of " + exp + " is not a valid multiplier");
    }
    return {
      pre,
      mul: multiplier
    };
  });
}
/**
 * creates a multiplier find table based on the "find" option
 *
 * @param find "find" option
 * @returns the multiplier find table from "find" option
 */
function createFindTable(find) {
  // return default table if "find" option is null or undefined
  if (find == null) {
    return transformItems(defaultBase1000ParseFindItems, 1000);
  }
  // use "find" option as base if it's a number
  if (isNumber(find)) {
    return transformItems(defaultBase1000ParseFindItems, find);
  }
  // use "find" option as items if it's an array
  if (isArray(find)) {
    return transformItems(find, 1000);
  }
  // get items and base from "find" option
  const {
    find: items,
    base
  } = find;
  const baseAsNumber = base !== null && base !== void 0 ? base : 1000;
  // use items if items provided
  if (items) {
    return transformItems(items, baseAsNumber);
  }
  // use default items
  return transformItems(defaultBase1000ParseFindItems, baseAsNumber);
}

/**
 * Creates a function which should return de multiplier based on captured unit
 *
 * @param find "find" option
 * @returns find multiplier function
 */
function createMulFinder(find) {
  // if "find" is a function
  if (isFunction(find)) {
    return (pre, unit) => {
      // find multiplier
      const result = find(pre, unit);
      // return null if no multiplier found (null | undefined)
      if (result == null) {
        return null;
      }
      if (isObject(result)) {
        throw error('Function returning object is no longer supported, return a non-zero number, null or undefined.');
      }
      if (!isNumber(result) || !isFiniteNumber(result) || result <= 0) {
        throw error(result + " is not a valid multiplier");
      }
      return result;
    };
  }
  const findTable = createFindTable(find);
  return pre => {
    // return 1 as multiplier if captured prefix is empty
    if (!pre) {
      return 1;
    }
    const item = findTable.find(_ref => {
      let {
        pre: prefix
      } = _ref;
      return prefix === pre;
    });
    // return null if not multiplier found
    if (!item) {
      return null;
    }
    const {
      mul: multiplier
    } = item;
    // return multiplier
    return multiplier;
  };
}

function createMatcher(matchOption) {
  if (isFunction(matchOption)) {
    return input => {
      const captured = matchOption(input);
      if (captured == null) {
        return null;
      }
      // throw if it's not an array
      if (!isArray(captured)) {
        throw error('match function should return an array of strings');
      }
      // return array
      return captured;
    };
  }
  const reg = matchOption == null ? /^\s*(-?\d*\.?\d*(?:e[+-]?\d+)?)\s*([a-z\u00b5]*)\s*$/i : new RegExp(matchOption);
  return input => {
    // execute RegExp against input
    const result = reg.exec(input);
    // if it doesn't match, return null
    if (!result) {
      return null;
    }
    // get a slice of the result array
    const resultAsArray = result.slice(1, 3);
    // return array
    return resultAsArray;
  };
}

function createParser(options) {
  if (options === void 0) {
    options = {};
  }
  if ('table' in options) {
    throw errorOptionRemoved('table', 'find');
  }
  const {
    unit,
    match: matchOption,
    find
  } = options;
  const extractPre = createExtractPre(unit);
  const findMultiplier = createMulFinder(find);
  const match = createMatcher(matchOption);
  return input => {
    // if input is of type number (number, NaN or Infinity)
    if (isNumber(input)) {
      // return the number if it's finite (number), otherwise (NaN or Infinity) return NaN
      return isFiniteNumber(input) ? input : NaN;
    }
    // return NaN if input is falsy (false, '', null or undefined) or true
    if (!input || input === true) {
      return NaN;
    }
    // convert input (probably non-empty string or object) to string
    const inputAsString = "" + input;
    // return NaN if value is empty string ('')
    if (!inputAsString) {
      return NaN;
    }
    // convert string to number
    const inputAsNumber = +inputAsString;
    // return number if string as number is finite (not NaN or Infinity)
    if (isFiniteNumber(inputAsNumber)) {
      return inputAsNumber;
    }
    // capture value & unit from string
    const captured = match(inputAsString);
    // return NaN if can't capture value & unit
    if (!captured) {
      return NaN;
    }
    const {
      length
    } = captured;
    if (length < 2) {
      throw error("match result array should have 2 items, got " + length);
    }
    // get value & unit from captured
    const [capturedValueAsString, wholeUnit] = captured;
    // return NaN if captured value is empty string ('')
    if (!capturedValueAsString) {
      return NaN;
    }
    // convert captured value to number
    const capturedValueAsNumber = +capturedValueAsString;
    // return NaN if value is not finite
    if (!isFiniteNumber(capturedValueAsNumber)) {
      return NaN;
    }
    const pre = extractPre(wholeUnit);
    // try to find find multiplier
    const multiplier = findMultiplier(pre, unit);
    // return NaN if can't find multiplier
    if (!multiplier) {
      return NaN;
    }
    // return value * multiplier
    return capturedValueAsNumber * multiplier;
  };
}

function parse(input, options) {
  return createParser(options)(input);
}

exports.MICRO = MICRO;
exports.createFormatter = createFormatter;
exports.createParser = createParser;
exports.format = format;
exports.parse = parse;
//# sourceMappingURL=gen-unit.cjs.map
